function Sounddemo(infile,Srate1)

%==========================================================================
% M-File:       SoundEnhance/SoundEnhance.m
% Functions:    Sounddemo       A software tool for animal speech analysis.
%
% Help Contents -----------------------------------------------------------
%   1. Basic description
%   2. Starting the program
%   3. Push buttons
%   4. Slider bar
%   5. Graph
%   6. Main menu
%       a. File
%       b. Display
%       c. Tools
%       d. Help
%   7. Primary enhancement functions
%
%
%
% 1. Brief description ------------------------------------------------------
%   A software tool with graphical interface for animal speech analysis. 
%   It provides capabilites for WAV wave graphing, enhancement, and
%   playback.  The interface is scalable, providing a consistent interface
%   for different resolutions. (1024x168 and up)
%   
%   Note:  only single-channel (mono) audio files are properly
%   handled by the program.  Stereo wave files cause the program to
%   misbehave.
%
%
%
% 2. Starting the program ----------------------------------------------------
%   - Make sure the SoundEnhance directory, which contains the m-files needed, 
%   is included in MatLab's Path ([MatLab 6.5] File -> Set Path...).
%   - To run this program, just type 'SoundEnhance' in the MATLAB window. 
%   This will open a file dialog box from which you can select the wave file 
%   to open.
%   - You may also type: "SoundEnhance filename", where 'filename' is the
%   name of the speech file.  This avoids the graphical dialog box.
%
%
%
% 3. Push buttons -----------------------------------------------------------
%   - Zoom In
%       Zooms in on the selected waveform between the cursors.
%       Click on a waveform...
%           Right-click: select zoom end.
%           Left-click: select zoom start.
%       Then, press the Zoom In button in the upper left corner of window.
%       Note:  Setting either start or end (but not both), graph defaults
%       to the start or end of the visible graph.  i.e. only right click to
%       set the end -> graph zooms in starting at the very front of waveform,
%       stopping where end cursor was set.
%       Note: see part 5 for information about the time and freq text boxes
%       display of either relative or absolute time.
%
%   - Zoom out
%       Zooms out to view the full waveform, resetting the cursors.
% 
%   - Play
%       Plays the region selected between the cursors.  The Zoom In button
%       is NOT needed to play a select portion of the wave.
%
%   - Set max samples to shift
%       Adjusts the scaling scroll bar by setting a new maximum. The text
%       box to the left of the button is where the desired maximum set.
%
%
% 4. Slider bar -------------------------------------------------------------
%   The slider bar, located just above the graph, is used during zooming.
%   It allows the wave to shift forward, maintaining the zoomed in view.  
%   To shift right:  left-click on the slider BOX.
%   To increase/decrease the amount of samples shifted, slide the slider
%   box right/left.  (farthest left = NO shift: farthest right = largest
%   shift)
%
%
%
% 5. Graph -----------------------------------------------------------------
%   - The main graph initially displays the wave with time versus
%   amplitude.  Use the main menu's Display submenu for other views (see
%   part 6 below for more info about the different types of graphs).
%   - The amplitude's visible range is dynamically adjusted to always fill
%   the y-axis (highest point touches the top, lowest point touches the
%   bottom).
%   - The 'time = #' and 'freq = #' (freq only visible during spectrogram)
%   show the current time or frequency of the point on the waveform where the 
%   mouse cursor is.  When zoomed in the x-axis uses relative time (always 
%   starting at 0), but the 'time' text box always uses absolute time, so 
%   you can see where the cursor is relative to the total waveform.
%
%
%
% 6. Main Menu --------------------------------------------------------------
%   Various graphs are made in different windows.  Help can be found in those
%   window's.
%   
%   a.  File
%       - Load: Loads a new wave into the chart (if two charts are open,
%       the currently selected wave is replaced).
%       - Load and Stack:  If only ONE file currently loaded, the loaded file 
%       does NOT replace the current file.  Instead, it is displaed above 
%       the original, with both files being displayed.
%       If TWO files are loaded already, Load and Stack acts just like a
%       regular Load.
%       - File Utility: A tool to do downsampling or upsampling to a
%       waveform.  See the Help button available within the tool.
%       - Print-Landscape and Print-Portrait:  Print to the default
%       printer.
%       - Print to File...:  Prints to either Postscript or Windows
%       metafile formats.  Useful when no printer is immediatly available.
%
%   b.  Display
%       - Time Waveform and Spectrogram:  Change the display of the main
%       window's graph.  Spectrogram allows choice of frequency range to
%       zoom in on.
%       - Energy Plot, F0 contour (Autocorrelation & Cepstrum), Formant
%       track, Power Spectral Density:  Each graph is displayed 
%       in its own separate window.
%       - Preferences: Allows choice of whether or not to view the cursors
%       on the graphs.
%
%   c.  Tools
%       Consists of various waveform enhancement algorithms.  Each option
%       creates a new wave file containing the resulting enhanced waveform.
%       - Spectral subtraction, Ephraim Malah log-spectral amplitude
%       filter, Iterative Wiener filtering, Wavelet denoising, Bandpass
%       digital filter, Lowpass digital filter.
%
%   d.  Help
%       Displays this help document in MatLab's help browser.  For reading
%       outside of MatLab, this text is located in
%       SoundEnhance/SoundEnhance.M
%
%
%
% 7. Primary enhancement functions ----------------------------------------
%   See the 'SoundEnhance Directory Contents' in SoundEnhance/Contents.m for
%   a listing of the primary signal enhancement functions used in the
%   program.
%   From MatLab console type: doc SoundEnhance/Contents.m
%
%==========================================================================


% Alan Chan
% Jan 28, 2005
%
% Jason Dudley
% August, 2005:     Added help documentation + minor improvements to
%                   interface.
%


%-- Variables --------------------------------------------------------------
colordef none  
global x xx
global Fs
global bits
global hl hr doit 
global fed shW
global filename
global fno Srate n_Secs filename AXISLOC  nLPC HDRSIZE
global nChannels fftSize filterWeights UpperFreq LowFreq ChanpUp
global center LPCSpec SpecUp Dur DurUp filterA filterB  S1 S0
global HDRSIZE cAxes En Be TIME centSMSP Asmsp Bsmsp OVRL WAV1 CLR
global TWOFILES wav agcsc MAX_AM upFreq upFreq2 FIX_SCALE
global SHOW_CRS SHOW_CHN ftype ftype2 bpsa bpsa2 PREEMPH LPC_ONLY
global lpcParam fft_par NAR_BAND SPEC_EXP FILT_TYPE VOL_MAX
global VOL_NORM
global sli n_samples FFT_SET XFFT
global tpc boc
global smp frq TOP
global preUp  defUp w64Up w128Up w256Up w512Up narUp
global crsUp chnUp lpcUp chnlpUp fbrd fnar
global ovlpfilt SET_X_AXIS LD_LABELS
global cursorExplaination
global sli
%==========================================================================


%-- First, get initial wave filename + screensize -------------------------
if (nargin < 1) % check for console command arguments
    [pth,fname] = dialogopen('open','*.wav');  % if none, use graphical file dialog.
    if ((~isstr(fname)) | ~min(size(fname))), return; end  
    filename=[pth,fname];
else    % otherwise, use console supplied file without graphical prompt.
    filename=infile;
end
pos = get(0, 'screensize'); % get the screensize
sWi = pos(3);
sHe = pos(4);

WIDTH   =round(0.99357*sWi);%original 0.9375
HEIGHT  =round(0.43*sHe) ;%originl 0.43
LEFT    =round(0.005*sWi);%original 0.025
BOTTOM  =round(sHe-HEIGHT-60);
%==========================================================================


%-- FLAGS -------------------------------------------------------------
LPCSpec = 1;            % if 0 display FFT, else LPC spectrum
TIME=1;                 % if 0 display spectrogram, else time waveform
WAV1=0;                 % If 1, then it is a .wav file with 8 bits/sample
CLR=1;                  % If 1 display spectrogram in color, else in gray scale
TWOFILES=0;             % If 1, then display two files
wav(1)=0; wav(2)=0;     % Used in case of dual-displays for 1-byte samples, as in WAV
upFreq=1.0;             % Upper frequency (percentage of Srate/2) in spectrogram
upFreq2=1.0;            % Upper frequency in spectrogram (1.0=Srate/2)
FIX_SCALE=-1;           % if > 0, then channel y-axis is always 0-15 dB
SHOW_CRS=1;             % if 1, show cursor lines, else dont
SHOW_CHN=1;             % if 1, show channel output/LPC display
PREEMPH=1;              % if 1, do pre-emphasis when computing the  spectrogram
LPC_ONLY=1;             % if 1, only the LPC specrtrum is displayed
lpcParam(1)=1;          % if 1, use hamming window in LPC analysis, else use rectangular 
lpcParam(2)=0;          % if 1, first-order pre-emphasis in LPC analysis, else dont
% NOTE: this pre-emphasis is done in addition to the CIS pre-emphasis
lpcParam(3)=-1;         % if 1, enhance spectral peaks in LPC analysis
fft_par(1)=1;           % if 1, use lines when plotting FFT, else use pickets
fft_par(2)=1;           % if 1 use hamming window, else rectangular
NAR_BAND=0;             % if 1, display narrowband spectrograms
SPEC_EXP=0.25;		% Used in spectrogram display (root compression)
FILT_TYPE='broad';	% the filter type
VOL_MAX=0;		% used for controling the volume
VOL_NORM=1;		% if 1, then volume is normalized
FFT_SET=0;		% if 1, use user defined FFT size
XFFT=0;			% same as above
SET_X_AXIS=0;
ovlpfilt=0;
LD_LABELS=0;
TOP=0;
%==========================================================================


%-- Opening wave file -----------------------------------------------------
fp = fopen(filename,'r');

if fp <=0
    disp('ERROR! File not found..')
    return;
end
ftype='short'; bpsa=2;    % bytes per sample
ftype2='short'; bpsa2=1;

ind1=find(filename == '.'); % Problem when there is a '.' in FOLDER path
if length(ind1)>1, ind=ind1(length(ind1)); else, ind=ind1; end;
ext = lower(filename(ind+1:length(filename))); 
[HDRSIZE, xSrate, bpsa, ftype] =  gethdr(fp,ext);
if xSrate==0, return; 
else Srate=xSrate; end;
if strcmp(ftype,'ascii')    
    x=fscanf(fp,'%f',inf);
else
    x  = fread(fp,inf,ftype);
end

fclose(fp); 
%==========================================================================


%-- Extract wave metadata ------------------------------------------------
[x,Fs,bits]=wavread(filename); % elephant (Reading .wav file data)
%==========================================================================


%-- Display wave's Frequency and set initial LPC order ---------------------
display (Fs);	
Srate = Fs;
%==========================================================================


%-- Fill dynamic range of graph (no impact on actual wave file) -----------
% x= x - mean(x);  %----------remove the DC bias----

MAX_AM=2048; % This allows a maximum resolution of __12-bits__

mx=max(x);
agcsc=MAX_AM/mx;
%==========================================================================


%-- Display basic wave characteristics ------------------------------------------
n_samples = length(x);  % Get number of samples
n_Secs    = n_samples/Fs;   % Get length in seconds
%=========================================================================


%-- Setting up some global variables --------------------------------------
Dur=10.0;   % Duration in msec of window
S1=n_samples;
S0=0;
Be=S0;
En=S1;
OVRL=1;  % if 1 then hold off plots, else hold on plots in 'pllpc'

fprintf('File: %s \nSamp.Freq: %d Hz,  num.samples: %d (%4.2f secs)\n\n',filename,Fs,n_samples,n_Secs); % Print wave metadata to console.
%==========================================================================


%-- Display main window --------------------------------------------------
fno =  figure('Units', 'Pixels', 'Position', [LEFT BOTTOM WIDTH HEIGHT],...
    'Resize','on','Name',strcat('SoundEnhance:_',filename),...
    'NumberTitle','Off','Resize','off','Menubar','None','Color','blue',...
    'WindowButtonDownFcn','mclick',...
    'KeyPressFcn','getkb');
set( fno, 'WindowButtonMotionFcn','showpt' );
%==========================================================================


%-- Determine the dimensions of the axis, then display graph --------------
le=round(0.2*WIDTH);
bo=round(0.13*HEIGHT);%0.174
wi=round(0.773*WIDTH);
he=round(0.739*HEIGHT);

AXISLOC = [le bo wi he];
cAxes = axes('Units','Pixels','Position',AXISLOC);
axes(cAxes);
Et=1000*n_samples/Fs;   % Et = time in msec
xax=0:1000/Fs:(Et-1000/Fs); % xaxis based on msec (not samples)
plot(xax,x,'y')
xlabel('Time (msecs)');
ylabel('Amplitude');
set(gca,'Color','k'); set(gca,'Xcolor','w'); set(gca,'YColor','w');
%set(gca,'Units','points','FontSize',9);
if min(x)<-1000 | mx >1000
    axis([0 Et min(x)-200 max(x)+200]);
else
    axis([0 Et min(x) max(x)]);
end

%set(gca,'Color','k');
set(gca,'XColor','white');   % x-axis color 
set(gca,'YColor','white');   % y-axis color
xywh = get(fno, 'Position');
axi=AXISLOC;
%==========================================================================


%-- Setup and display buttons and scroll bar ------------------------------
left = 10;
wide = 80;
top  = xywh(4) - 10;
high = 22;
high=22;
if 9*(22+8) > xywh(4), high=17; end;
inc  = high + 8;

% Slider bar
defaultShiftAmount = 1000;
sli = uicontrol('Style','slider','min',0,'max',defaultShiftAmount,'Callback','getslide',...
    'Position',[axi(1)+100 axi(2)+axi(4)+14 axi(3)/2 14], 'Tag','sliTag');
shiftBarTitle = uicontrol('Style','text','Position',[axi(1)+(axi(3)/4) axi(2)+axi(4)+30 80 13],...
    'HorizontalAlignment', 'center','String','Window Shift','BackGroundColor',[0 0 1],...
    'FontWeight','bold');
shiftMin = uicontrol('Style','text','Position',[axi(1) axi(2)+axi(4)+14 100 14],...
    'String','Minimum (no shift)','BackGroundColor',[0 0 1],'HorizontalAlignment','left');
shiftMax = uicontrol('Style','text','Position',[axi(1)+100+(axi(3)/2)+10 axi(2)+axi(4)+14 70 14],...
    'String','Maximum','BackGroundColor',[0 0 1],'HorizontalAlignment','left');
sliMaxText = uicontrol( 'Style','edit','String',defaultShiftAmount,...
    'Position',[axi(1)+100+(axi(3)/2)+80 axi(2)+axi(4)+14 60 16],...
    'HorizontalAlignment', 'left', 'Tag', 'sliMaxTextTag' );
sliSetButton = uicontrol( 'Style','pushbutton','String','Set Max Samples To Shift',...
    'Position',[axi(1)+100+(axi(3)/2)+80+70 axi(2)+axi(4)+14 130 16], ...
    'HorizontalAlignment', 'left',...
    'Callback','sliderAdjustRatio( findobj(''Tag'',''sliMaxTextTag''), findobj(''Tag'',''sliTag'') )' );

% Zoom In button
Zin = uicontrol('Style', 'PushButton', 'Callback','zoomi(''in'')', ...
    'HorizontalAlign','center', 'String', 'Zoom In',...
    'Position', [left top-high wide high]);

% Zoom Out button
top = top - inc;
Zout = uicontrol('Style', 'PushButton', 'Callback','zoomi(''out'')', ...
    'HorizontalAlign','center', 'String', 'Zoom Out',...
    'Position', [left top-high wide high]);

%-- Set initial LPC order ----------
if Fs>12000
    nLPC=14;
else
    nLPC=12; % initialize LPC order
end
%-----------------------------------

% Play button - plays selecte part of wave (between the cursors)
top = top - inc-20;
plUp = uicontrol('Style', 'PushButton', 'Callback', 'playf(''sel'')', ...
    'HorizontalAlign','center', 'String', 'Play',...
    'Position', [left top-high wide/2 high]);
%==========================================================================


%-- Draw the squares (used to select the active graph) in case there are
% TWOFILES displayed ------------------------------------------------------
wwi=xywh(3); whe=xywh(4);
tpc=uicontrol('Style','text','Position',[120 2*whe/3+10 45 15],'String',' ','BackGroundColor',[0 0 1]);
boc=uicontrol('Style','text','Position',[120 whe/3-10   45 15],'String',' ','BackGroundColor',[0 0 1]);
%==========================================================================


%-- Draw the time and freq numbers ----------------------------------------
smp=uicontrol('Style','text','Position',[10 33 wide+40 16],'BackGroundColor',[0 0 0],...
    'HorizontalAlignment','left','FontWeight','bold');
frq=uicontrol('Style','text','Position',[10 10 wide+40 16],'BackGroundColor',[0 0 0],...
    'HorizontalAlignment','left','FontWeight','bold');
%=========================================================================


%-- MENUS ----------------------------------------------------------------
%-- File menu item -------------------------------------------------------
ff=uimenu('Label','File');
uimenu(ff,'Label','&Load','Callback','loadfile(''replace'')');
uimenu(ff,'Label','&Load and stack','Callback','loadfile(''stack'')');
uimenu(ff,'Label','File utility','Callback','filetool');
uimenu(ff,'Label','Print-Landscape','Callback','cprint(''landscape'',''printer'')','Separator','on');
uimenu(ff,'Label','Print-Portrait','Callback','cprint(''portrait'',''printer'')');
fprf=uimenu(ff,'Label','Print to file ...');
uimenu(fprf,'Label','Postscript','Callback','cprint(''landscape'',''eps'')');
uimenu(fprf,'Label','Windows metafile','Callback','cprint(''landscape'',''meta'')');


uimenu(ff,'Label','Exit','CallBack','quitall','Separator','on');

%-- Display menu item -----------------------------------------------------
fd=uimenu('Label','Display');
uimenu(fd,'Label','Time Waveform','Callback','setdisp(''time'')');
fd0= uimenu(fd,'Label','Spectrogram');
uimenu(fd0,'Callback','setdisp(''spec'',''clr'')',...
    'Label','Color');
uimenu(fd0,'Callback','setdisp(''spec'',''noclr'')',...
    'Label','Gray Scale');
uimenu(fd0,'Callback','setdisp(''spec'',''1khz'')',...
    'Label','0-1 kHz');
uimenu(fd0,'Callback','setdisp(''spec'',''3khz'')',...
    'Label','0-3 kHz');
uimenu(fd0,'Callback','setdisp(''spec'',''6khz'')',...
    'Label','0-6 kHz');
uimenu(fd0,'Callback','setdisp(''spec'',''10khz'')',...
    'Label','0-10 kHz');
uimenu(fd0,'Callback','setdisp(''spec'',''full'')',...
    'Label','Full Range: 0-fs/2');

uimenu(fd,'Label','Energy Plot','Callback','engy');
fdf0=uimenu(fd,'Label','F0 contour');
uimenu(fdf0,'Label','Autocorrelation approach','Callback','estf0(''autocor'')');
uimenu(fdf0,'Label','Cepstrum approach','Callback','estf0(''cepstrum'')');
uimenu(fd,'Label','Formant track','Callback','ftrack(''plot'')');
uimenu(fd,'Label','Power Spectral Density','Callback','estpsd');
fd2=uimenu(fd,'Label','Preferences');
crsUp=uimenu(fd2,'Label','  Show Cursor Lines','Checked','on',...
    'Callback','prefer(''crs'')');

%-- Tools menu item -------------------------------------------------------
fm1=uimenu('Label','Tools');

uimenu(fm1,'Label','Spectral subtraction','CallBack','enhance(''ss1'')');
uimenu(fm1,'Label','Ephraim Malah log-spectral amplitude filter','CallBack','enhance(''em1'')');
uimenu(fm1,'Label','Iterative Wiener filtering','CallBack','enhance(''wf1'')');
uimenu(fm1,'Label','Wavelet Denoising','CallBack','enhance(''wd1'')');
uimenu(fm1,'Label','Bandpass digital filter','CallBack','enhance(''bpf1'')');
uimenu(fm1,'Label','Lowpass digital filter','CallBack','enhance(''lpf1'')');

%-- Help menu item -------------------------------------------------------- 
uimenu('Label','Help','Callback','helpf(''Sounddemo'')');
%==========================================================================


%----------- Initialize handles to cursor lines ---------------------------
np=3; Ylim=get(gca,'YLim');
hl=line('Xdata',[np np],'Ydata',Ylim,'Linestyle','-',...
    'color',[0 0 0],'Erasemode','xor');

hr=line('Xdata',[np np],'Ydata',Ylim,'Linestyle','--',...
    'color',[0 0 0],'Erasemode','xor');
doit=0;
%==========================================================================


%-- Cursor Explaination text ----------------------------------------------
cursorExplaination = uicontrol('Style','text','Position',[10 (2*whe/3+10)-70 115 50],...
    'horizontalAlignment', 'left','String','CURSORS:            Right mouse=zoom end. Left mouse=zoom start + data collection start',...
    'BackGroundColor',[0 0 1]);
%==========================================================================

