function yy=iwf(x,Fs,win_len,iternum,ALPHA_dB,NP,P,G_MIN_dB, nf)
%  wf=iwf(x,Fs,L,10,   10,8,16,-20);
%  wf=iwf(x,Fs,L,10,   10,8,16,-10); % elephant

%
% function yy=wienerfilt(x,Fs,iternum,ALPHA_dB,NP,P,G_MIN_dB)
% Performs unconstrained iterative speech enhancement
% based on the MAP estimate of the original signal.
% AR model coefficients for noise power spectrum estimate are used. 
%
% x: Noisy signal
% iternum: Number of iterations
% ALPHA_dB: Noise suppression factor
% NP: Noise spectrum AR order[0-8] the higher value means more
% resolution in frequency. For broadband noise a value of 0-4 is 
% suggested, whereas for narrowband noise the order may go up to 8.
% P: Prediction order for noise AR
% nf: number of frames used to estimate the noise spectrum - Modified by Alan
% 
% yy: Enhanced signal
%
%  Written by: Levent Arslan           June 11, 1993
%  Modified by Jidong Tao              August 2004
%  Modified by Alan Chan               April 2005
%  
win_len = win_len*nf;   % first nf frame will be used
if nargin<8
    G_MIN_dB=-10;
end
if nargin<7
    P=ceil(Fs/1000);           % PREDICTION ORDER FOR SPEECH AR SPECTRUM
end
if nargin<6
    NP=ceil(Fs/2000);          % PREDICTION ORDER FOR NOISE AR SPECTRUM
end
% if nargin < 4
%     ALPHA_dB = 0;
% end
% if nargin < 3
%     iternum=2;
% end
% win_len = Fs/40; % WINDOW SIZE
ALPHA=10^(ALPHA_dB/10); % suppression factor
new_ALPHA = ALPHA;
% x=filter([1 -0.95],1,x);
G_MIN = 10^(G_MIN_dB/20); % min floor of wiener filter freq response
BETA = 0.5; % POWER OF WIENER FILTER
sm_counter=0;
% gamma = 1.0; % DURBIN COEFFICIENTS CONSTANT
gam_present = 0.7; % WEIGHT OF PRESENT AUTOCORRELATION LAG
gam_past = 1 - gam_present; % WEIGHT OF PAST AUTOCORRELATION LAG

x=x(:); n=length(x); % Get signal length

w=0:2*pi/win_len:(win_len-1)/win_len*pi; % FREQUENCY BINS, dim: 1x(L/2)
w_matrix=ones(P,1)*w; % dim: Px(L/2)
w_matrix2=ones(max(NP,1),1)*w; % NPx(L/2)
one=ones(size(w));
exp_term = exp(-j*([1:P]'*ones(1,win_len/2)).*w_matrix); % exp(jkw) terms in Ps(w), dim:Px(L/2)
exp_term2 =exp(-j*([1:max(NP,1)]'*ones(1,win_len/2)).*w_matrix2); % exp(jkw) terms in Pn(w), dim:NPx(L/2)

% NOISE POWER SPECTRUM(Pn(w)) AND ENERGY  ESTIMATION FROM THE SILENCE FRAMES
%Pn_w=ones(win_len,1)+abs(fft(x(1:win_len).*hanning(win_len))); % only use 1st nf frame to estimate

L = win_len/nf;
overlap=L/2; % let overlap be 0.5 of window length
num_frames=floor(n/(L-overlap)+1); % # of frames of signal

%Noise spectrum estimation from the silence frames of data.
% The number of frames nf - modified by Alan Chan
%    num_win_noise= nf; % Using first fn frames of x to be reference noise
%    for i=1:num_win_noise
%       An(:,i)=x(1+(i-1)*overlap : L+(i-1)*overlap); % An dim Lxframe
%    end
%    Noise = abs(fft(An.*(hanning(L)*ones(1,num_win_noise)))); % magnitude of noise dim Lxframe
%    clear An
%    Pn_w=(Noise'); % noise, dim 1xL


% Estimate the noise from the first nf of the data
num_n = fix (length (x)/(nf*L)) - 1; 
num_n_rem = rem(length(x), (nf *L));
for k = 1: num_n + 1
    nos((k-1)*nf*L+ 1: k*(nf*L)) = x(1: nf*L);
end    

nos((k)*(nf*L)+ 1 :( k)*(nf*L) + num_n_rem ) = x(1: num_n_rem);

denom_Pn_w = fft (nos(1:win_len/2));


RN=autocorr(x(1:win_len).*hanning(win_len),NP);
% aN=gamma*durbin(RN);

aN=real(lpc(x(1:win_len).*hanning(win_len),NP)'); % LPC coefficients of noise 
aN=aN(2:NP+1); % cut off a0, keep a(1)~a(NP+1)
%denom_Pn_w = one ./ (abs(one + aN'*exp_term2).^2); % 1/|(1+a*exp(-jwi))^2|, dim:1x(L/2)
noise_gain = RN(1) - sum(aN.*RN(2:NP+1));
noise_en = noise_gain * sum(denom_Pn_w);
Pn_w = max(100*ones(1,win_len/2),noise_gain * denom_Pn_w); % Noise spec. from AR model



x(n+1:n+2*win_len)=zeros(2*win_len,1);  % ADD ZEROS AT THE END OF THE SPEECH SIGNAL
overlap=win_len/2; % 50 PERCENT OVERLAP AMONG FRAMES
num_frames=floor(n/(win_len-overlap)+1); % NUMBER OF FRAMES
% disp(sprintf('Number of frames = %d',num_frames)),

for i=1:num_frames
    frame=x((i-1)*(win_len-overlap)+1:(i-1)*(win_len-overlap)+win_len);
    deg(:,i)=frame.*hanning(win_len); % HANNING WINDOWING
end

for iter=1:iternum
    x(n+1:n+2*win_len)=zeros(2*win_len,1); % ADD ZEROS AT THE END OF THE SPEECH SIGNAL
    for i=1:num_frames
        frame=x((i-1)*(win_len-overlap)+1:(i-1)*(win_len-overlap)+win_len);
        y(:,i)=frame.*hanning(win_len); % HANNING WINDOWING
    end
    
    for i=1:num_frames
        R(:,i) = max(rand(P+1,1),autocorr(y(:,i),P));  
        if iter > 1
            R(:,i) = gam_present*R(:,i) + gam_past*prev_R(:,i); % APPLY SMOOTHING
        end                                                    % ACROSS ITERATIONS
        a=durbin(R(:,i));
        denom_Ps_w = one ./ (abs(one - a'*exp_term).^2);
        lpc_energy = sum(denom_Ps_w);
        speech_en=(R(1,i)-sum(a.*R(2:P+1,i)))*lpc_energy;
        g_2=max(speech_en - noise_en, noise_en/2)/lpc_energy; % ESTIMATE GAIN
        Ps_w=2*g_2*denom_Ps_w; % USE THE GAIN IN CLEAN SPEECH SPECTRUM ESTIMATE
        Py_w=speech_en/lpc_energy*denom_Ps_w;
       
        
        display (length(Ps_w));
        display (length(Pn_w));
                
        
        wien_fil=(Ps_w ./(Ps_w + new_ALPHA*Pn_w)).^BETA; 
              
        wien_fil(win_len/2+1:win_len)=wien_fil(win_len/2:-1:1);
        wien_fil=(wien_fil>G_MIN).*wien_fil + G_MIN*(wien_fil <= G_MIN);
        enh_sp_fft=wien_fil' .* fft(deg(:,i));
        y(:,i)=real(ifft(enh_sp_fft));
    end
    % RECONSTRUCTION(HANNING)
    y(1:win_len-overlap)=y(1:win_len-overlap,1);
    for i=1:num_frames-1
        yy(i*(win_len-overlap)+1:i*(win_len-overlap)+win_len-overlap)=y(win_len-overlap+1:win_len,i)+y(1:win_len-overlap,i+1);
    end
    yy=yy(1:n);
    prev_R = R; % SAVE PRESENT AUTOCORRELATION LAGS TO USE IN THE NEXT ITERATION  
end
yy=yy(:);