function x = logstsa(y,L,nf,alpha, n);

% yarmal - Speech Enhancement by Log-spectral Amplitude Estimation
%          (by Y.Ephraim & D.Malah)
% ================================================================
% Usage:
%       x = yarmal(y,n,[alpha],[clean]);
% Inputs:
%        y     - noisy signal.
%        n     - reference noise.
%        alpha - the smoothing constant required by the algorithm [0.98]. 
% Outputs:
%         x     - enhanced signal.

% constants
% =========
win_len     = L;    % length of hanning window.
overlap_len = L/2;    % amount of non overlapping.
if (nargin == 2)
  alpha = 0.98;
end

% Estimate the noise from the first nf of the data
num_n = fix (length (y)/(nf*L)) - 1; 

num_n_rem = rem(length(y), (nf *L));
for k = 1: num_n 
    n((k-1)*nf*L+ 1: k*(nf*L)) = y(1: nf*L);
end    
%n(k*(nf*L)+ 1 : k*(nf*L) + num_n_rem) = y(1: num_n_rem);
    


%Noise spectrum estimation from the silence frames of data.
%if nargin== 4
 %   num_win_noise= nf; % Using first n frames of x to be reference noise
  %  for k=1:num_win_noise
  %     An(:,k)=y(1+(k-1)*overlap_len:L+(k-1)*overlap_len); % An dim L*frame
  % end
  %  Noise=abs(fft(An.*(hanning(L)*ones(1,num_win_noise)))); % magnitude of noise dim L*frame
   % clear An

   % n=mean(real(ifft(Noise'))); % average noise, dim 1xL
   %end


% adjusting dimensions of y and n 
% ===============================
y = y(:)';
n = n(:)';

% truncating the data to length of N*256 if necessary
% ===================================================
data_len = length(y);   % number of noisy signal samples.
noise_len = length(n);  % number of noise samples.

delta_data  = rem(data_len, win_len);
delta_noise = rem(noise_len,win_len);

if (delta_data ~= 0)
   %disp('Truncating noisy signal to N*L samples')
   half_delta = fix(delta_data/2);
   y = y(1 + half_delta : (data_len - (delta_data - half_delta)));
   data_len = data_len - delta_data;
end

if (delta_noise ~= 0)
   %disp('Truncating noise to N*L samples')
   half_delta = fix(delta_noise/2);
   n = n(1 + half_delta : (noise_len - (delta_noise - half_delta)));
   noise_len = noise_len - delta_noise;
end

% calculating number of overlapping windows
% =========================================
x = zeros(1,data_len);  % initialize enhanced signal
num_win = 1 + (data_len - win_len)/overlap_len;
num_win_noise = 1 + (noise_len - win_len)/overlap_len;

% lambdaD calculation
% ===================
for I=1:num_win_noise
   An(:,I)=n(1+(I-1)*overlap_len:win_len + (I-1)*overlap_len)';
end

N = fft(An.*(hanning(win_len)*ones(1,num_win_noise)));
clear An
lambdaD = mean((N.*conj(N))');
clear N

% Main Loop
% =========
%disp(['total number of frames: ' int2str(num_win)]);
disp(' ');

for I=1:num_win
   if (mod(I,10)==0)
 %     disp([num2str(I),' frames done'])
   end
   Ay = y(1+(I-1)*overlap_len : win_len + (I-1)*overlap_len)';
   Y  = fft(Ay.*hanning(win_len));
   gamaK = Y.*conj(Y)./lambdaD';
   if I==1,
      % Initial estimation of apriori SNR
      % =================================
      ksi=alpha+(1-alpha)*p(gamaK-1);
      TEMP=ksi./(ksi+1);
      Agal=TEMP.*exp(0.5*ei(TEMP.*gamaK)).*abs(Y);
      %Agal=TEMP.*exp(0.5*ei_ap(TEMP.*gamaK)).*abs(Y);
   else
      % Estimation of apriori SNR
      % =========================
      ksi=alpha*Agal.*Agal./lambdaD'+(1-alpha)*p(gamaK-1);  
      TEMP=ksi./(ksi+1);
      % Estimation of the log-spectral amp. estimator
      % =============================================
      Agal=TEMP.*exp(0.5*ei(TEMP.*gamaK)).*abs(Y);
      %Agal=TEMP.*exp(0.5*ei_ap(TEMP.*gamaK)).*abs(Y);
   end
   % Adding the noisy phase
   % ======================
   ygal=Agal.*exp(i*angle(Y));
   % Transformation to Time-Domain
   % =============================
   xtag=ifft(ygal);
   x((1+(I-1)*overlap_len):(win_len+(I-1)*overlap_len))=...
   x((1+(I-1)*overlap_len):(win_len+(I-1)*overlap_len))+...
   xtag';
end
x = real(x);
x = x(:);
%disp(' ')

return
