function y = spec_sub(x,L, nf,N);
% function y = spec_sub(x,L,N)
% Speech Enahnced using spectral subtraction based on a short time
% with overlapping frames. 
%
% y: enhanced speech signal
% nf:  frame number to estimate noise
% x: noisy speech signal
% L: window size
% N: reference noise
%
% Related  programs: psychoacoustic
% Written by: Levent Arslan     June 2, 1993
%
% Modified by Jidong Tao        July 2004
% Modified by Alan Chan         April  2005


n=length(x); % size of noisy signal
x=x(:); % column noisy speech, dim nx1
x(n+1:n+4*L)=zeros(4*L,1); % add 4L zeros after noisy signal: dim (n+4L)x1
overlap=L/2; % let overlap be 0.5 of window length
num_frames=floor(n/(L-overlap)+1); % # of frames of signal

%Noise spectrum estimation from the silence frames of data.
% The number of frames nf - modified by Alan Chan
if nargin < 4
    num_win_noise= nf; % Using first fn frames of x to be reference noise
    for i=1:num_win_noise
       An(:,i)=x(1+(i-1)*overlap : L+(i-1)*overlap); % An dim Lxframe
    end
    Noise = abs(fft(An.*(hanning(L)*ones(1,num_win_noise)))); % magnitude of noise dim Lxframe
    clear An
    N=mean(Noise'); % average noise, dim 1xL
end

% Computation of FFT(magnitude and phase) for each window
for i=1:num_frames
   frame=x((i-1)*(L-overlap)+1:(i-1)*(L-overlap)+L); % dim Lx1
   frame=frame.*hanning(L); %Triangular windowing, dim Lx1
   frame_fft=fft(frame); % dim Lx1
   ang_frame(:,i)=angle(frame_fft); % dim L x num_frames
   mag_frame(:,i)=abs(frame_fft); % dim L x num_frames
end

% Spectral Subtraction with Half-Wave rectification
for i=1:num_frames
   enhanced_mag=mag_frame(:,i)-N'; % dim Lx1
% Phase information from noisy speech is added to enhanced speech
   fft_y=enhanced_mag.*cos(ang_frame(:,i))+j*enhanced_mag.*sin(ang_frame(:,i));
   time_y(:,i)=real(ifft(fft_y)); % dim L x num_frames
end

% Reconstruction(Hanning)
y(1:L-overlap)=time_y(1:L-overlap,1);
for i=1:num_frames-1
   y(i*(L-overlap)+1:i*(L-overlap)+L-overlap)=time_y(L-overlap+1:L,i)+time_y(1:L-overlap,i+1);
end
y=y(1:n); % dim nx1
%y=real(max(x)/max(real(y))*y);
y=y(:);